/*	**************************************************************************************
    *									overlay.js
    *									==========
    *
    *	Author:	Geva Zeichner
	*	Forum:	http://forums.mozillazine.org/viewtopic.php?t=408043
    *	Date:	15/04/2006
    *	Edited:	04/06/2006
    **************************************************************************************/

window.addEventListener("load", function(e){IMDb_Preview.handle_load(e);}, false);

var IMDb_Preview =
{
	/*************************************************************************************
	 *	Handle_Load(e)
	 *	==============
	 *	Listeners, defs, inits...
	 *************************************************************************************/
	handle_load: function(e)
	{
		// Shhh... listen...
		window.addEventListener("unload", function(e){IMDb_Preview.handle_unload(e);}, false);
		window.addEventListener("mouseover", function(e){IMDb_Preview.handle_mouse_over(e);}, false);
		window.addEventListener("mousemove", function(e){IMDb_Preview.handle_mouse_move(e);}, false);
		window.addEventListener("mousedown", function(e){IMDb_Preview.terminate_preview();}, false);
		
		// Consts
		this.TITLE_REGEXP = /^((http:\/\/)(\w*\.)?((imdb\.com)(.*)(\/title\/tt\d{7})(\/)?))$/;
		this.ACTOR_REGEXP = /^((http:\/\/)(\w*\.)?((imdb\.com)(.*)(\/name\/nm\d{7})(\/)?))$/;
		this.IMG_REGEXP = /^((http:\/\/ia\.imdb\.com\/media)(.*\.)(jpg)?(gif)?(png)?)$/;
		this.MOVIE_TYPE = 1;
		this.ACTOR_TYPE = 2;
		this.NOT_SUBMITTED_IMG_PATH = "chrome://imdb_preview/skin/not_submitted.png";
		this.LOADING_IMG_PATH = "chrome://imdb_preview/skin/loading.gif";
		this.LOADING_BG_PATH = "chrome://imdb_preview/skin/loading_bg.png";
		this.E404_IMG_PATH = "chrome://imdb_preview/skin/error_404.png";
		this.ERROR_IMG_PATH = "chrome://imdb_preview/skin/error.png";
		
		// Set variables
		this.prefs = Components.classes['@mozilla.org/preferences-service;1'].
						getService(Components.interfaces.nsIPrefService).
						getBranch('extensions.imdb_preview.');
		this.move_timer = "";
		this.mouse_over_url_target = "";
		this.loaded_url = " ";
		this.last_owner_doc = "";
		this.http_req = new XMLHttpRequest();
		this.is_showing = false;
		this.lock_processing = false;
		this.page_x = 0;
		this.page_y = 0;
		
		this.check_timer = "";
		this.poster_timeout = "";
		
		this.loading_start_time = 0;
		this.loading_end_time = 0;
		this.actual_loading_time = 0;
		this.complete_loading_time = 0;
		this.poster_loading_time = 0;
		this.total_loading_time = 0;
		
		// Init preview settings //
		
		this.loading_bg_img = window._content.document.createElement("img");
		this.loading_bg_img.src = this.LOADING_BG_PATH;
		this.loading_bg_img.id = "imdb_preview_item";
		
		this.loading_img = window._content.document.createElement("img");
		this.loading_img.src = this.LOADING_IMG_PATH;
		this.loading_img.setAttribute("style", "-moz-opacity: 0.5; position: relative; \
											    left: 8px; top: -31px;");
		this.loading_img.id = "imdb_preview_item";
		
		this.loading_div = window._content.document.createElement("div");
		this.loading_div.appendChild(this.loading_bg_img);
		this.loading_div.appendChild(this.loading_img);
		this.loading_div.style.width = "38px";
		this.loading_div.style.height = "38px";
		this.loading_div.id = "imdb_preview_item";
		
		this.poster_img = window._content.document.createElement("img");
		this.poster_img.addEventListener("load", function(e){IMDb_Preview.handle_img_load(e);}, false);
		this.poster_img.src = "about:blank";
		this.poster_img.id = "imdb_preview_item";
		
		this.wrap_div = window._content.document.createElement("div");
		this.wrap_div.appendChild(this.loading_div);
		this.wrap_div.id = "imdb_preview_item";
		
		// Poster style settings
		this.wrap_div.style.display = "none";
		this.wrap_div.style.textTop = "true";
		this.wrap_div.style.zIndex = "1000";
		this.wrap_div.style.position = "absolute";
		this.wrap_div.style.left = "0px";
		this.wrap_div.style.top = "0px";
	},
	
	/*************************************************************************************
	 *	Handle_Unload()
	 *	===============
	 *	Cleanups...
	 *************************************************************************************/
	handle_unload: function()
	{
		if (this.prefs.prefHasUserValue("actual_loading_time"))
		{
			this.prefs.clearUserPref("actual_loading_time");
			this.prefs.clearUserPref("complete_loading_time");
			this.prefs.clearUserPref("poster_loading_time");
		}
		
		// used until v0.2.1
		if (this.prefs.prefHasUserValue("halt_time"))
		{
			this.prefs.clearUserPref("halt_time");
		}
	},
	
	/*************************************************************************************
	 *	Handle_Mouse_Over(e)
	 *	====================
	 *	Gets info about what we stepped into
	 *************************************************************************************/
	handle_mouse_over: function(e)
	{
		this.page_x = e.pageX;
		this.page_y = e.pageY;
		this.mouse_over_url_target = unescape(e.target.href);
		this.last_owner_doc = e.originalTarget.ownerDocument;
		
		if (!unescape(e.target.id).match("imdb_preview_item"))
		{
			this.terminate_preview();
		}
	},
	
	/*************************************************************************************
	 *	Handle_Mouse_Move(e)
	 *	====================
	 *	Checks if we are halting somewhere
	 *************************************************************************************/
	handle_mouse_move: function(e)
	{
		clearTimeout(this.move_timer);
		
		var movie_delay = this.prefs.getIntPref("movie_delay_time");
		var actor_delay = this.prefs.getIntPref("actor_delay_time");
		
		// If it's a movie link
		if ((this.mouse_over_url_target.match(this.TITLE_REGEXP)) &&
			(movie_delay >= 0))
			this.move_timer = setTimeout(function(){IMDb_Preview.handle_mouse_halt();},
										 movie_delay);
		
		// If it's an actor link
		else if ((this.mouse_over_url_target.match(this.ACTOR_REGEXP)) &&
				 (actor_delay >= 0))
			this.move_timer = setTimeout(function(){IMDb_Preview.handle_mouse_halt();},
										 actor_delay);
	},
	
	/*************************************************************************************
	 *	Handle_Mouse_Halt()
	 *	===================
	 *	Checks if we halted on the right spot
	 *************************************************************************************/
	handle_mouse_halt: function()
	{
		// If standing on url that is different from the one stood on before
		if (this.mouse_over_url_target != this.loaded_url)
		{
			// If it's a movie link
			if (this.mouse_over_url_target.match(this.TITLE_REGEXP))
			{
				// Load url
				this.load_target(this.mouse_over_url_target, this.MOVIE_TYPE);
			}
			
			// If it's an actor link
			else if (this.mouse_over_url_target.match(this.ACTOR_REGEXP))
			{
				// Load url
				this.load_target(this.mouse_over_url_target, this.ACTOR_TYPE);
			}
		}
		this.loaded_url = this.mouse_over_url_target;
	},
	
	/*************************************************************************************
	 *	Load_Target(url_target, target_type)
	 *	====================================
	 *	Retrives the target's html code and triggers its processing
	 *************************************************************************************/
	load_target: function(url_target, target_type)
	{
		this.is_showing = true;
		this.dump("\nLoading...\n");
		
		this.loading_end_time = 0;
		this.actual_loading_time = 0;
		this.complete_loading_time = 0;
		this.poster_loading_time = 0;
		this.total_loading_time = 0;
		
		this.loading_start_time = new Date();
		
		// Show 'loading...' poster
		this.load_img(this.LOADING_IMG_PATH);
		
		// Load target
		this.http_req.open('GET', url_target, true);
		
		// Get check html prefs
		var start_check = this.prefs.getIntPref("start_check");
		var stop_check = this.prefs.getIntPref("stop_check");
		var check_intervals = this.prefs.getIntPref("check_intervals");
		
		// Try processing before loading completed
		this.check_timer = window.setInterval(
			function ()
			{
				// Get curr time
				var curr_time = new Date();
				
				// Calculate how much time passed
				curr_time = curr_time - IMDb_Preview.loading_start_time;
				
				// If above start and below stop
				if ((curr_time >= start_check) && (curr_time <= stop_check))
					
					// Process
					IMDb_Preview.process_response(IMDb_Preview.http_req.responseText,
												  target_type);
				// If above stop
				else if (curr_time > stop_check)
					
					// Stop
					window.clearInterval(IMDb_Preview.check_timer);
			},
			check_intervals);
		
		this.http_req.onreadystatechange = function()
		{
			// If loading completed
			if (IMDb_Preview.http_req.readyState == 4)
			IMDb_Preview.timer = setTimeout( function ()
			{
				// Stop check html
				window.clearInterval(IMDb_Preview.check_timer);
				
				try
				{
					// With no error
					if (IMDb_Preview.http_req.status == 200)
					{
						var curr_date = new Date();
						
						IMDb_Preview.process_response(IMDb_Preview.http_req.responseText,
													  target_type);
						
						IMDb_Preview.complete_loading_time =
							curr_date - IMDb_Preview.loading_start_time;
						IMDb_Preview.dump("complete: " +
										  IMDb_Preview.complete_loading_time + " ms\n");
						IMDb_Preview.prefs.setIntPref("complete_loading_time",
													  IMDb_Preview.complete_loading_time);
					}
					else
					{
						var curr_date = new Date();
						IMDb_Preview.loading_end_time = curr_date;
						IMDb_Preview.dump("Error loading page\n");
						IMDb_Preview.load_img(IMDb_Preview.E404_IMG_PATH);
					}
				}
				catch(err)
				{
					IMDb_Preview.dump(err);
				}
			}, 0);
		};
		this.http_req.send(null);
	},
	
	/*************************************************************************************
	 *	Process_Response(html_source, target_type)
	 *	==========================================
	 *************************************************************************************/
	process_response: function(html_source, target_type)
	{
		// If processing allowed
		if (this.lock_processing == false)
		{
			// Process what we got
			var result = this.process_html(html_source, target_type);
			
			// If got a result
			if (result != -1)
			{
				// Don't allow further html processing
				this.lock_processing = true;
				
				// Stop check html
				window.clearInterval(this.check_timer);
				
				var curr_date = new Date();
				this.loading_end_time = curr_date;
				
				// If poster not submitted
				if (result == -2)
				{
					// Load N/S poster
					this.load_img(this.NOT_SUBMITTED_IMG_PATH);
					this.dump("Poster Not Submitted\n\n");
				}
				// If found poster
				else if (result.match(this.IMG_REGEXP))
				{
					// Load poster
					this.load_img(result);
					this.dump(result + "\n\n");
				}
				else
				{
					this.load_img(IMDb_Preview.ERROR_IMG_PATH);
					this.dump("Unknown error. Notify extension author.\nResult:\n" + result);
				}
			}
		}
	},
	
	/*************************************************************************************
	 *	Process_HTML(html_source, target_type)
	 *	======================================
	 *************************************************************************************/
	process_html: function(html_source, target_type)
	{
		var result = -1;
		
		// If the target is a movie page
		if (target_type == this.MOVIE_TYPE)
		{
			// Process...
			result = this.process_movie_html(html_source);
		}
		// If the target is an actor page
		else if (target_type == this.ACTOR_TYPE)
		{
			// Process...
			result = this.process_actor_html(html_source);
		}
		
		// If got a result
		if (result != -1)
		{
			// Get title
			var title_start_pos = html_source.indexOf('<title>') + 7;
			var title_end_pos = html_source.indexOf('</title>', title_start_pos);
			var title_name = html_source.substring(title_start_pos, title_end_pos);
			this.dump(title_name + ":\n");
		}
		
		return(result);
	},
	
	/*************************************************************************************
	 *	Process_Movie_HTML(html_source)
	 *	===============================
	 *************************************************************************************/
	process_movie_html: function(html_source)
	{
		var img_src = -1;
		var poster_pos, img_src_start_pos, img_src_end_pos;
		
		// Try to find a poster
		poster_pos = html_source.indexOf('<a name="poster"');
		if (poster_pos != -1)
		{
			img_src_start_pos = html_source.indexOf('src="', poster_pos) + 5;
			if (img_src_start_pos != -1)
			{
				img_src_end_pos = html_source.indexOf('" height', img_src_start_pos);
				if (img_src_end_pos != -1)
				{
					// Get IMG's src
					img_src = html_source.substring(img_src_start_pos, img_src_end_pos);
				}
			}
		}
		
		// If poster not found yet
		if (img_src == -1)
		{
			// Maybe poster not submitted
			if (html_source.indexOf('title="Poster Not Submitted"') != -1)
			{
				img_src = -2;
			}
		}
		
		return(img_src);
	},
	
	/*************************************************************************************
	 *	Process_Actor_HTML(html_source)
	 *	===============================
	 *************************************************************************************/
	process_actor_html: function(html_source)
	{
		var img_src = -1;
		var poster_pos, img_src_start_pos, img_src_end_pos;
		
		// Try to find a poster
		poster_pos = html_source.indexOf('<a name="headshot"');
		if (poster_pos != -1)
		{
			img_src_start_pos = html_source.indexOf('src="', poster_pos) + 5;
			if (img_src_start_pos != -1)
			{
				img_src_end_pos = html_source.indexOf('" width', img_src_start_pos);
				if (img_src_end_pos != -1)
				{
					// Get IMG's src
					img_src = html_source.substring(img_src_start_pos, img_src_end_pos);
				}
			}
		}
		
		// If poster not found yet
		if (img_src == -1)
		{
			// Maybe poster not submitted
			if (html_source.indexOf('alt="Photo Not Submitted"') != -1)
			{
				img_src = -2;
			}
		}
		
		return(img_src);
	},
	
	/*************************************************************************************
	 *	Load_IMG(img_src)
	 *	=================
	 *	Loads an image into the img object
	 *************************************************************************************/
	load_img: function(img_src)
	{
		if (this.is_showing == true)
		{
			this.last_owner_doc.body.appendChild(this.wrap_div);
			
			if (img_src == this.LOADING_IMG_PATH)
			{
				this.wrap_div.appendChild(this.loading_div);
				this.show_preview(38, 38);
			}
			else
			{
				this.poster_img.style.display = "none";
				this.wrap_div.appendChild(this.poster_img);
				this.poster_img.src = img_src;
				window.clearTimeout(this.poster_timeout);
				this.poster_timeout = window.setTimeout(
					function()
					{
						IMDb_Preview.load_img(IMDb_Preview.E404_IMG_PATH);
						IMDb_Preview.dump("Poster loading timed out.\n");
					},
					this.prefs.getIntPref("poster_timeout"));
			}
		}
	},
	
	/*************************************************************************************
	 *	Handle_IMG_Load(e)
	 *	==================
	 *	When the img finished to load
	 *************************************************************************************/
	handle_img_load: function(e)
	{
		if (this.poster_img.src != "about:blank")
		{
			window.clearTimeout(this.poster_timeout);
			this.wrap_div.style.display = "none";
			this.wrap_div.removeChild(this.loading_div);
			this.show_preview(this.poster_img.height, this.poster_img.width);
			
			// Stats
			var curr_date = new Date();
			
			this.actual_loading_time = this.loading_end_time - this.loading_start_time;
			this.poster_loading_time = curr_date - this.loading_end_time;
			this.total_loading_time = this.actual_loading_time + this.poster_loading_time;
			
			this.dump("actual: " + this.actual_loading_time + " ms\n" +
					  "poster: " + this.poster_loading_time + " ms\n" +
					  "total: " + this.total_loading_time + " ms\n");
			
			this.prefs.setIntPref("actual_loading_time", this.actual_loading_time);
			this.prefs.setIntPref("poster_loading_time", this.poster_loading_time);
		}
	},
	
	/*************************************************************************************
	 *	Show_Preview(img_height, img_width)
	 *	===================================
	 *	Positions the preview and reveals it
	 *************************************************************************************/
	show_preview: function(img_height, img_width)
	{
		var doc = this.last_owner_doc;
		var top, left = 0;
		
		// If in upper half of window
		if (this.page_y < (doc.defaultView.innerHeight / 2) + doc.defaultView.pageYOffset)
			
			// Display preview below cursor
			top = this.page_y + 32;
		
		// If in lower half of window
		else
			top = this.page_y - img_height - 10;
		
		// If in the left half of window
		if (this.page_x < (doc.defaultView.innerWidth / 2) + doc.defaultView.pageXOffset)
			
			// Display preview to the right of the cursor
			left = this.page_x + 10;
		
		// If in the right half of window
		else
			left = this.page_x - img_width - 10;
		
		this.wrap_div.style.top = top + "px";
		this.wrap_div.style.left = left + "px";
		
		// Reveals the preview
		this.wrap_div.style.display = "block";
		this.poster_img.style.display = "block";
	},
	
	/*************************************************************************************
	 *	Terminate_Preview()
	 *	===================
	 *	I'll be back...
	 *************************************************************************************/
	terminate_preview: function()
	{
		if (this.is_showing == true)
		{
			window.clearInterval(this.check_timer);
			window.clearTimeout(this.poster_timeout);
			this.is_showing = false;
			this.lock_processing = false;
			this.wrap_div.appendChild(this.poster_img);
			this.wrap_div.removeChild(this.poster_img);
			this.wrap_div.appendChild(this.loading_div);
			this.poster_img.src = "about:blank";
			this.loaded_url = " ";
			this.wrap_div.style.display = "none";
			this.http_req.abort();
		}
	},
	
	/*************************************************************************************
	 *	Dump(text)
	 *	==========
	 *************************************************************************************/
	dump: function(text)
	{
		if (this.prefs.getBoolPref("dump") == true)
		{
			dump(text);
		}
	}
};